@@ -71,30 +71,35 @@
 #' cdf(X, quantile(X, 0.7))
 #' quantile(X, cdf(X, 7))
 Exponential <- function(rate = 1) {
-  d <- list(rate = rate)
+  d <- data.frame(rate = rate)
   class(d) <- c("Exponential", "distribution")
   d
 }
 
-#' @export
-print.Exponential <- function(x, ...) {
-  cat(glue("Exponential distribution (rate = {x$rate})"), "\n")
-}
-
 #' @export
 mean.Exponential <- function(x, ...) {
   ellipsis::check_dots_used()
-  x$rate ^ -1
+  rval <- x$rate^-1
+  setNames(rval, names(x))
 }
 
 #' @export
-variance.Exponential <- function(x, ...) x$rate ^ 2
+variance.Exponential <- function(x, ...) {
+  rval <- x$rate^2
+  setNames(rval, names(x))
+}
 
 #' @export
-skewness.Exponential <- function(x, ...) 2
+skewness.Exponential <- function(x, ...) {
+  rval <- rep.int(2, length(x))
+  setNames(rval, names(x))
+}
 
 #' @export
-kurtosis.Exponential <- function(x, ...) 6
+kurtosis.Exponential <- function(x, ...) {
+  rval <- rep.int(6, length(x))
+  setNames(rval, names(x))
+}
 
 #' Draw a random sample from an Exponential distribution
 #'
@@ -102,14 +107,22 @@ kurtosis.Exponential <- function(x, ...) 6
 #'
 #' @param x An `Exponential` object created by a call to [Exponential()].
 #' @param n The number of samples to draw. Defaults to `1L`.
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #' @param ... Unused. Unevaluated arguments will generate a warning to
 #'   catch mispellings or other possible errors.
 #'
-#' @return A numeric vector of length `n`.
+#' @return In case of a single distribution object or `n = 1`, either a numeric
+#'   vector of length `n` (if `drop = TRUE`, default) or a `matrix` with `n` columns
+#'   (if `drop = FALSE`).
 #' @export
 #'
-random.Exponential <- function(x, n = 1L, ...) {
-  rexp(n = n, rate = x$rate)
+random.Exponential <- function(x, n = 1L, drop = TRUE, ...) {
+  n <- make_positive_integer(n)
+  if (n == 0L) {
+    return(numeric(0L))
+  }
+  FUN <- function(at, d) rexp(n = at, rate = x$rate)
+  apply_dpqr(d = x, FUN = FUN, at = n, type = "random", drop = drop)
 }
 
 #' Evaluate the probability density function of an Exponential distribution
@@ -119,21 +132,28 @@ random.Exponential <- function(x, n = 1L, ...) {
 #' @param d An `Exponential` object created by a call to [Exponential()].
 #' @param x A vector of elements whose probabilities you would like to
 #'   determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{dexp}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-pdf.Exponential <- function(d, x, ...) {
-  dexp(x = x, rate = d$rate)
+pdf.Exponential <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dexp(x = at, rate = d$rate, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "density", drop = drop)
 }
 
 #' @rdname pdf.Exponential
 #' @export
 #'
-log_pdf.Exponential <- function(d, x, ...) {
-  dexp(x = x, rate = d$rate, log = TRUE)
+log_pdf.Exponential <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dexp(x = at, rate = d$rate, log = TRUE)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "logLik", drop = drop)
 }
 
 #' Evaluate the cumulative distribution function of an Exponential distribution
@@ -143,14 +163,20 @@ log_pdf.Exponential <- function(d, x, ...) {
 #' @param d An `Exponential` object created by a call to [Exponential()].
 #' @param x A vector of elements whose cumulative probabilities you would
 #'   like to determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{pexp}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-cdf.Exponential <- function(d, x, ...) {
-  pexp(q = x, rate = d$rate)
+cdf.Exponential <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) pexp(q = at, rate = d$rate, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "probability", drop = drop)
 }
 
 #' Determine quantiles of an Exponential distribution
@@ -161,15 +187,22 @@ cdf.Exponential <- function(d, x, ...) {
 #' @inheritParams random.Exponential
 #'
 #' @param probs A vector of probabilities.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{qexp}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of quantiles, one for each element of `probs`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(probs)` columns (if `drop = FALSE`). In case of a vectorized
+#'   distribution object, a matrix with `length(probs)` columns containing all
+#'   possible combinations.
 #' @export
 #'
-quantile.Exponential <- function(x, probs, ...) {
+quantile.Exponential <- function(x, probs, drop = TRUE, ...) {
   ellipsis::check_dots_used()
-  qexp(p = probs, rate = x$rate)
+  FUN <- function(at, d) qexp(at, rate = x$rate, ...)
+  apply_dpqr(d = x, FUN = FUN, at = probs, type = "quantile", drop = drop)
 }
 
 #' Fit an Exponential distribution to data
@@ -208,14 +241,17 @@ suff_stat.Exponential <- function(d, x, ...) {
 #' Return the support of the Exponential distribution
 #'
 #' @param d An `Exponential` object created by a call to [Exponential()].
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #'
 #' @return A vector of length 2 with the minimum and maximum value of the support.
 #'
 #' @export
-support.Exponential <- function(d){
-  if(!is_distribution(d)){
-    message("d has to be a disitrubtion")
-    stop()
-  }
-  return(c(0, Inf))
+support.Exponential <- function(d, drop = TRUE) {
+  stopifnot("d must be a supported distribution object" = is_distribution(d))
+  stopifnot(is.logical(drop))
+
+  min <- rep(0, length(d))
+  max <- rep(Inf, length(d))
+
+  make_support(min, max, d, drop = drop)
 }
