@@ -68,32 +68,36 @@
 #'
 #' cdf(X, quantile(X, 0.7))
 #' quantile(X, cdf(X, 7))
-#'
 Poisson <- function(lambda) {
-  d <- list(lambda = lambda)
+  d <- data.frame(lambda = lambda)
   class(d) <- c("Poisson", "distribution")
   d
 }
 
-#' @export
-print.Poisson <- function(x, ...) {
-  cat(glue("Poisson distribution (lambda = {x$lambda})"), "\n")
-}
-
 #' @export
 mean.Poisson <- function(x, ...) {
   ellipsis::check_dots_used()
-  x$lambda
+  rval <- x$lambda
+  setNames(rval, names(x))
 }
 
 #' @export
-variance.Poisson <- function(x, ...) x$lambda
+variance.Poisson <- function(x, ...) {
+  rval <- x$lambda
+  setNames(rval, names(x))
+}
 
 #' @export
-skewness.Poisson <- function(x, ...) 1 / sqrt(x$lambda)
+skewness.Poisson <- function(x, ...) {
+  rval <- 1 / sqrt(x$lambda)
+  setNames(rval, names(x))
+}
 
 #' @export
-kurtosis.Poisson <- function(x, ...) 1 / x$lambda
+kurtosis.Poisson <- function(x, ...) {
+  rval <- 1 / x$lambda
+  setNames(rval, names(x))
+}
 
 #' Draw a random sample from a Poisson distribution
 #'
@@ -101,14 +105,22 @@ kurtosis.Poisson <- function(x, ...) 1 / x$lambda
 #'
 #' @param x A `Poisson` object created by a call to [Poisson()].
 #' @param n The number of samples to draw. Defaults to `1L`.
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #' @param ... Unused. Unevaluated arguments will generate a warning to
 #'   catch mispellings or other possible errors.
 #'
-#' @return A numeric vector of length `n`.
+#' @return In case of a single distribution object or `n = 1`, either a numeric
+#'   vector of length `n` (if `drop = TRUE`, default) or a `matrix` with `n` columns
+#'   (if `drop = FALSE`).
 #' @export
 #'
-random.Poisson <- function(x, n = 1L, ...) {
-  rpois(n = n, lambda = x$lambda)
+random.Poisson <- function(x, n = 1L, drop = TRUE, ...) {
+  n <- make_positive_integer(n)
+  if (n == 0L) {
+    return(numeric(0L))
+  }
+  FUN <- function(at, d) rpois(n = at, lambda = d$lambda)
+  apply_dpqr(d = x, FUN = FUN, at = n, type = "random", drop = drop)
 }
 
 #' Evaluate the probability mass function of a Poisson distribution
@@ -118,21 +130,28 @@ random.Poisson <- function(x, n = 1L, ...) {
 #' @param d A `Poisson` object created by a call to [Poisson()].
 #' @param x A vector of elements whose probabilities you would like to
 #'   determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{dpois}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-pdf.Poisson <- function(d, x, ...) {
-  dpois(x = x, lambda = d$lambda)
+pdf.Poisson <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dpois(x = at, lambda = d$lambda, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "density", drop = drop)
 }
 
 #' @rdname pdf.Poisson
 #' @export
 #'
-log_pdf.Poisson <- function(d, x, ...) {
-  dpois(x = x, lambda = d$lambda, log = TRUE)
+log_pdf.Poisson <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dpois(x = at, lambda = d$lambda, log = TRUE)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "logLik", drop = drop)
 }
 
 #' Evaluate the cumulative distribution function of a Poisson distribution
@@ -142,14 +161,20 @@ log_pdf.Poisson <- function(d, x, ...) {
 #' @param d A `Poisson` object created by a call to [Poisson()].
 #' @param x A vector of elements whose cumulative probabilities you would
 #'   like to determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{ppois}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-cdf.Poisson <- function(d, x, ...) {
-  ppois(q = x, lambda = d$lambda)
+cdf.Poisson <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) ppois(q = at, lambda = d$lambda, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "probability", drop = drop)
 }
 
 #' Determine quantiles of a Poisson distribution
@@ -160,15 +185,22 @@ cdf.Poisson <- function(d, x, ...) {
 #' @inheritParams random.Poisson
 #'
 #' @param probs A vector of probabilities.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{qpois}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of quantiles, one for each element of `probs`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(probs)` columns (if `drop = FALSE`). In case of a vectorized
+#'   distribution object, a matrix with `length(probs)` columns containing all
+#'   possible combinations.
 #' @export
 #'
-quantile.Poisson <- function(x, probs, ...) {
+quantile.Poisson <- function(x, probs, drop = TRUE, ...) {
   ellipsis::check_dots_used()
-  qpois(p = probs, lambda = x$lambda)
+  FUN <- function(at, d) qpois(p = at, lambda = d$lambda, ...)
+  apply_dpqr(d = x, FUN = FUN, at = probs, type = "quantile", drop = drop)
 }
 
 #' Fit an Poisson distribution to data
@@ -208,14 +240,17 @@ suff_stat.Poisson <- function(d, x, ...) {
 #' Return the support of the Poisson distribution
 #'
 #' @param d An `Poisson` object created by a call to [Poisson()].
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #'
 #' @return A vector of length 2 with the minimum and maximum value of the support.
 #'
 #' @export
-support.Poisson <- function(d){
-  if(!is_distribution(d)){
-    message("d has to be a disitrubtion")
-    stop()
-  }
-  return(c(0, Inf))
+support.Poisson <- function(d, drop = TRUE) {
+  stopifnot("d must be a supported distribution object" = is_distribution(d))
+  stopifnot(is.logical(drop))
+
+  min <- rep(0, length(d))
+  max <- rep(Inf, length(d))
+
+  make_support(min, max, d, drop = drop)
 }
