@@ -113,40 +113,54 @@
 #' mean(x) + quantile(T, 0.12 / 2) * sd(x) / sqrt(nx)
 #' mean(x) + quantile(T, 1 - 0.12 / 2) * sd(x) / sqrt(nx)
 StudentsT <- function(df) {
-  d <- list(df = df)
+  d <- data.frame(df = df)
   class(d) <- c("StudentsT", "distribution")
   d
 }
 
-#' @export
-print.StudentsT <- function(x, ...) {
-  cat(glue("Student's T distribution (df = {x$df})"), "\n")
-}
-
 #' @export
 mean.StudentsT <- function(x, ...) {
   ellipsis::check_dots_used()
-  if (x$df > 1) 0 else NaN
+  rval <- ifelse(x$df > 1,
+    0,
+    NaN
+  )
+  setNames(rval, names(x))
 }
 
 #' @export
 variance.StudentsT <- function(x, ...) {
-  if (x$df > 2) x$df / (x$df - 2)
-  else if (x$df > 1) Inf
-  else NaN
+  rval <- ifelse(x$df > 2,
+    x$df / (x$df - 2),
+    ifelse(x$df > 1,
+      Inf,
+      NaN
+    )
+  )
+  setNames(rval, names(x))
 }
 
 #' @export
-skewness.StudentsT <- function(x, ...) if(x$df > 3) 0 else NaN
+skewness.StudentsT <- function(x, ...) {
+  rval <- ifelse(x$df > 3,
+    0,
+    NaN
+  )
+  setNames(rval, names(x))
+}
 
 #' @export
 kurtosis.StudentsT <- function(x, ...) {
-  if (x$df > 4) 6 / (x$df - 4)
-  else if (x$df > 2) Inf
-  else NaN
+  rval <- ifelse(x$df > 4,
+    6 / (x$df - 4),
+    ifelse(x$df > 2,
+      Inf,
+      NaN
+    )
+  )
+  setNames(rval, names(x))
 }
 
-
 #' Draw a random sample from a StudentsT distribution
 #'
 #' Please see the documentation of [StudentsT()] for some properties
@@ -157,17 +171,24 @@ kurtosis.StudentsT <- function(x, ...) {
 #'
 #' @param x A `StudentsT` object created by a call to [StudentsT()].
 #' @param n The number of samples to draw. Defaults to `1L`.
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #' @param ... Unused. Unevaluated arguments will generate a warning to
 #'   catch mispellings or other possible errors.
 #'
 #' @family StudentsT distribution
 #'
-#' @return A numeric vector of length `n`.
+#' @return In case of a single distribution object or `n = 1`, either a numeric
+#'   vector of length `n` (if `drop = TRUE`, default) or a `matrix` with `n` columns
+#'   (if `drop = FALSE`).
 #' @export
 #'
-#'
-random.StudentsT <- function(x, n = 1L, ...) {
-  rt(n = n, df = x$df)
+random.StudentsT <- function(x, n = 1L, drop = TRUE, ...) {
+  n <- make_positive_integer(n)
+  if (n == 0L) {
+    return(numeric(0L))
+  }
+  FUN <- function(at, d) rt(n = at, df = d$df)
+  apply_dpqr(d = x, FUN = FUN, at = n, type = "random", drop = drop)
 }
 
 #' Evaluate the probability mass function of a StudentsT distribution
@@ -181,23 +202,30 @@ random.StudentsT <- function(x, n = 1L, ...) {
 #' @param d A `StudentsT` object created by a call to [StudentsT()].
 #' @param x A vector of elements whose probabilities you would like to
 #'   determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{dt}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
 #' @family StudentsT distribution
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-pdf.StudentsT <- function(d, x, ...) {
-  dt(x = x, df = d$df)
+pdf.StudentsT <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dt(x = at, df = d$df, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "density", drop = drop)
 }
 
 #' @rdname pdf.StudentsT
 #' @export
 #'
-log_pdf.StudentsT <- function(d, x, ...) {
-  dt(x = x, df = d$df, log = TRUE)
+log_pdf.StudentsT <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) dt(x = at, df = d$df, log = TRUE)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "logLik", drop = drop)
 }
 
 #' Evaluate the cumulative distribution function of a StudentsT distribution
@@ -207,16 +235,22 @@ log_pdf.StudentsT <- function(d, x, ...) {
 #' @param d A `StudentsT` object created by a call to [StudentsT()].
 #' @param x A vector of elements whose cumulative probabilities you would
 #'   like to determine given the distribution `d`.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{pt}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
 #' @family StudentsT distribution
 #'
-#' @return A vector of probabilities, one for each element of `x`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(x)` columns (if `drop = FALSE`). In case of a vectorized distribution
+#'   object, a matrix with `length(x)` columns containing all possible combinations.
 #' @export
 #'
-cdf.StudentsT <- function(d, x, ...) {
-  pt(q = x, df = d$df)
+cdf.StudentsT <- function(d, x, drop = TRUE, ...) {
+  FUN <- function(at, d) pt(q = at, df = d$df, ...)
+  apply_dpqr(d = d, FUN = FUN, at = x, type = "probability", drop = drop)
 }
 
 #' Determine quantiles of a StudentsT distribution
@@ -236,27 +270,41 @@ cdf.StudentsT <- function(d, x, ...) {
 #' @inheritParams random.StudentsT
 #'
 #' @param probs A vector of probabilities.
-#' @param ... Unused. Unevaluated arguments will generate a warning to
-#'   catch mispellings or other possible errors.
+#' @param drop logical. Should the result be simplified to a vector if possible?
+#' @param ... Arguments to be passed to \code{\link[stats]{qt}}.
+#'   Unevaluated arguments will generate a warning to catch mispellings or other
+#'   possible errors.
 #'
-#' @return A vector of quantiles, one for each element of `probs`.
+#' @return In case of a single distribution object, either a numeric
+#'   vector of length `probs` (if `drop = TRUE`, default) or a `matrix` with
+#'   `length(probs)` columns (if `drop = FALSE`). In case of a vectorized
+#'   distribution object, a matrix with `length(probs)` columns containing all
+#'   possible combinations.
 #' @export
 #'
 #' @family StudentsT distribution
 #'
-quantile.StudentsT <- function(x, probs, ...) {
+quantile.StudentsT <- function(x, probs, drop = TRUE, ...) {
   ellipsis::check_dots_used()
-  qt(p = probs, df = x$df)
+  FUN <- function(at, d) qt(p = at, df = x$df, ...)
+  apply_dpqr(d = x, FUN = FUN, at = probs, type = "quantile", drop = drop)
 }
 
 
 #' Return the support of the StudentsT distribution
 #'
 #' @param d An `StudentsT` object created by a call to [StudentsT()].
+#' @param drop logical. Should the result be simplified to a vector if possible?
 #'
 #' @return A vector of length 2 with the minimum and maximum value of the support.
 #'
 #' @export
-support.StudentsT <- function(d){
-  return(c(-Inf, Inf))
+support.StudentsT <- function(d, drop = TRUE) {
+  stopifnot("d must be a supported distribution object" = is_distribution(d))
+  stopifnot(is.logical(drop))
+
+  min <- rep(-Inf, length(d))
+  max <- rep(Inf, length(d))
+
+  make_support(min, max, d, drop = drop)
 }
